/* v2text.c: read on stdin a user supplied text description of the BIOS jump table
plus data area (2048 byte beginning of physical memory), and write the binary
image of the area to a binary file. The user may optionally supply the name 
for the output file.

In both formats the characters in the file are hexadecimal digits, lower or
upper case. 

Format 1: 2 blocks of 80x24 lines of characters, plus an odd-sized block,
separated by CR/LF characters ( = output of bblurt.bin )

Format 2: Lines of 0,4, or 8 hex digits, terminated by CR/LF. In the 8 digit
instance the digits represent a single jump table entry, a double word pair 
giving the segment:offset of the handler routine. In the 4 character instance
it is understood that the missing 4 should match those of the previous line.
In the 0 character instance it is understood that all 8 characters match 
those of the previous line. (This rule may be applied iteratively.) This
format relieves (somewhat) the tedium of recording the output of bblurt2.bin
by hand.)

Format 2 is the default. Format 1 may be selected with an option.

This should only be run under 16 bit DOS since we assume that a char is
16 bit, etc.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

typedef unsigned char byte;

#define NN 2048      /* Number of bytes in the jump table */
#define NVECTORS 512 /* NN/4 */
#define DEF_FN "VECTOR.BIN"    /* Default file to write */
#define MAX_FN 32
#define LINE_LENGTH 80
#define SCREEN_LINES 24   /* same as bblurt.bin to prevent loss of first
			     line through scrolling */
#define MY_NAME "text2v"
#define VERSION "1.0"
#define USAGE "text2v [-1 -h -v] [filename]"
#define HELP "-v: give version number and exit.\n" 
#define HELP1 "-h: print this helpful information and exit.\n"
#define HELP2 "-1: input in bblurt.bin format (2 full pages and odd lines.)\n"
#define HELP3 "filename: path to binary to write. Default: vector.bin.\n"
#define HELP4 "Reads its text input on stdin. Redirect as needed.\n"

static char hexdgs[] = "0123456789ABCDEF";
static byte vbuf[NN];                              
static char vfname[MAX_FN];

int main(int argc, char **argv){

	FILE *out;
	int usebb2_format = 1;
	int i=0,j=0,n=0,ln;
	byte c,hi,lo;
	byte y,z;
	char linebuf0[256]; /* holds last full line */
	char linebuf1[256]; /* holds current line */

	/* Default file name to open */
	strcpy(vfname,DEF_FN);

	/* Process command line */
	while(++j < argc){
		if((argv[j][0] == '-')||(argv[j][0]=='/'))
			switch(argv[j][1]){ 
				case 'v':
				case 'V':
					printf("%s\n",VERSION);
					exit(0);
				case '?':
				case 'h':
				case 'H':
					printf("%s\n",USAGE);
					printf("%s%s%s%s%s",HELP,HELP1,HELP2,
					HELP3,HELP4);
					exit(0);
				case '1':
					usebb2_format = 0;
					continue;
				default:
					fprintf(stderr,"%s: unkown option %s\n",
						MY_NAME,argv[j]);
					exit(1);
			}
		else {     /* Not option, must be filename to open */
			strcpy(vfname,argv[j]);  
			break;
		}
	}

	if(j+1 < argc){
			fprintf(stderr,"%s\n",USAGE);
			exit(1);
	}
		
	out = fopen(vfname,"wb");
	if(!out){
		fprintf(stderr,"%s: could not open %s for binary write\n",
		MY_NAME,vfname);
		return 1;
	}
	printf("%s: opened %s for binary write. \n",MY_NAME,vfname);
	if(usebb2_format){

	n = 0; /* count lines */
	j = 0; /* count bytes */
	while(gets(linebuf1) != NULL){
		n++;
		/* Get next line and complete as needed */

		for(i=0;i<8;i++)if(linebuf1[i]=='\0')linebuf1[i]=linebuf0[i];
		linebuf1[i]='\0';
		strcpy(linebuf0,linebuf1); /* store last complete line */

		/* loop over the 8 digits, decode and store each pair 
		   in binary buffer */
		for(i=0;i<8;){
			hi = linebuf1[i++];
			y = 0;
			if(islower(hi)) hi = toupper(hi);
			switch(hi){
				case '0':
					break;
				case '1':
					y = 1;
					break;
				case '2':
					y = 2;
					break;
				case '3' :
					y = 3;
					break;
				case '4':
					y = 4;
					break;
				case '5':
					y = 5;
					break;
				case '6':
					y = 6;
					break;
				case '7':
					y = 7;
					break;
				case '8':
					y = 8;
					break;
				case '9':
					y = 9;
					break;
				case 'A':
					y = 10;
					break;
				case 'B':
					y = 11;
					break;
				case 'C':
					y = 12;
					break;
				case 'D':
					y = 13;
					break;
				case 'E':
					y = 14;
					break;
				case 'F':
					y = 15;
					break;
				default:
					fprintf(stderr,"%s: got bad digit(%d), line %d, digit %d\n",
						MY_NAME,hi,n,i);
					break;
				 
			}
			lo = linebuf1[i++];
			if(islower(lo)) lo = toupper(lo); 
			z = 16*y;
			y = 0;
			switch(lo){     
				case '0':
					break;
				case '1':
					y = 1;
					break;
				case '2':
					y = 2;
					break;
				case '3' :
					y = 3;
					break;
				case '4':
					y = 4;
					break;
				case '5':
					y = 5;
					break;
				case '6':
					y = 6;
					break;
				case '7':
					y = 7;
					break;
				case '8':
					y = 8;
					break;
				case '9':
					y = 9;
					break;
				case 'A':
					y = 10;
					break;
				case 'B':
					y = 11;
					break;
				case 'C':
					y = 12;
					break;
				case 'D':
					y = 13;
					break;
				case 'E':
					y = 14;
					break;
				case 'F':
					y = 15;
					break;
				default:
					fprintf(stderr,"%s: got bad digit (%d), line %d, digit %d\n",
						MY_NAME,lo,n,i);
					break;
				 
			}

			z += y;
			vbuf[j++]= z;
		}
		if(n > NVECTORS){
			fprintf(stderr,"%s: read too many lines: %d \n",MY_NAME,n);
			exit(1);
		}

	}
	}
	else {  /* using bblurt.bin type output */
	n = 0; /* count lines */
	j = 0; /* count bytes */
	while(gets(linebuf1) != NULL){          
		n++;
		ln = strlen(linebuf1);
		if(ln == 0)continue; /* skip blank lines */
		/* loop over the digits, decode and store each pair 
		   in binary buffer */
		for(i=0;i<ln;){
			hi = linebuf1[i++];
			y = 0;
			if(islower(hi)) hi = toupper(hi);
			switch(hi){
				case '0':
					break;
				case '1':
					y = 1;
					break;
				case '2':
					y = 2;
					break;
				case '3' :
					y = 3;
					break;
				case '4':
					y = 4;
					break;
				case '5':
					y = 5;
					break;
				case '6':
					y = 6;
					break;
				case '7':
					y = 7;
					break;
				case '8':
					y = 8;
					break;
				case '9':
					y = 9;
					break;
				case 'A':
					y = 10;
					break;
				case 'B':
					y = 11;
					break;
				case 'C':
					y = 12;
					break;
				case 'D':
					y = 13;
					break;
				case 'E':
					y = 14;
					break;
				case 'F':
					y = 15;
					break;
				default:
					fprintf(stderr,"%s: got bad high digit (%d), line %d, digit %d\n",
						MY_NAME,hi,n,i);
					break;
				 
			}
			lo = linebuf1[i++];
			if(islower(lo)) lo = toupper(lo); 
			z = 16*y;
			y = 0;
			switch(lo){     
				case '0':
					break;
				case '1':
					y = 1;
					break;
				case '2':
					y = 2;
					break;
				case '3' :
					y = 3;
					break;
				case '4':
					y = 4;
					break;
				case '5':
					y = 5;
					break;
				case '6':
					y = 6;
					break;
				case '7':
					y = 7;
					break;
				case '8':
					y = 8;
					break;
				case '9':
					y = 9;
					break;
				case 'A':
					y = 10;
					break;
				case 'B':
					y = 11;
					break;
				case 'C':
					y = 12;
					break;
				case 'D':
					y = 13;
					break;
				case 'E':
					y = 14;
					break;
				case 'F':
					y = 15;
					break;
				default:
					fprintf(stderr,"%s: got bad low digit (%d), line %d, digit %d hi=%c\n",
						MY_NAME,lo,n,i,hi);
					break;
				 
			}

			z += y;
			vbuf[j++]= z;
		}
	}
	}
	printf("%s: read %d lines %d bytes\n",MY_NAME,n,j);
	if(usebb2_format && (n < NVECTORS)){                             
		fprintf(stderr,"%s: did not read sufficiently many entries. Exiting\n",
		MY_NAME);
		return 1;
	}
	/* Write buffer out to file */
	fwrite(vbuf,sizeof(byte),NN,out);
	fclose(out);
	return 0;
}

